open Jane.Std
open Core_extended.Std
open Lz4lib.Lz4_ffi


let str1 = "There are six CAMLparam macros: CAMLparam0 to CAMLparam5, which take
  zero to five arguments respectively. If your function has fewer than 5 parameters of
  type value, use the corresponding macros with these parameters as arguments. If your
  function has more than 5 parameters of type value, use CAMLparam5 with five of these
  parameters, and use one or more calls to the CAMLxparam macros for the remaining
  parameters (CAMLxparam1 to CAMLxparam5)."

let str2 = "The compressed stream is composed of sequences.
  Each sequence starts with a token. The token is a one byte value, separated into two
  4-bits fields (which therefore range from 0 to 15). The first field is to indicate the
  length of literals. If it is 0, then there is no literal. If it is 15, then we need to add
  some more bytes to indicate the full length. Each additionnal
  byte then represent a value of 0 to 255, which is added to the previous value to produce a
  total length. When the byte value is 255, another byte is output. There can be any number
  of bytes following the token."

let str3 = "The size of the hash table can be modified while respecting full format
  compatibility. For restricted memory systems, this is an important feature, since the
  hash size can be reduced to 15 bits, or 12 bits, or even 10 bits (1024 positions,
  needing only 4K). Obviously, the smaller the table, the more collisions (false positive)
  we get, reducing compression power. But it nonetheless still works, and remain fully
  compatible with more complex and memory-hungry versions. The decoder do not care of the
  method used to find matches, and requires no additional memory.
  The fast version of LZ4 hosted on Google Code uses a fast scan strategy, which is a
  single- cell wide hash table. Each position in the input stream gets hashed, using the first 4
  bytes (minmatch). Then the position is stored at the hashed position.
  The size of the hash table can be modified while respecting full format compatibility. For
  restricted memory systems, this is an important feature, since the hash size can be
  reduced to 15 bits, or 12 bits, or even 10 bits (1024 positions, needing only
  4K). Obviously, the smaller the table, the more collisions (false positive) we get,
  reducing compression power. But it nonetheless still works, and remain fully compatible
  with more complex and memory-hungry versions. The decoder do not care of the method used
  to find matches, and requires no additional memory."

let bigstr1 = Bigstring.of_string str1
let bigstr2 = Bigstring.of_string str2
let bigstr3 = Bigstring.of_string str3

let check_string_compression source =
  let isize = String.length source in
  let dest = Lz4_string.make_compression_dest source in
  let result_str = String.create isize in

  let compressed_size = Lz4_string.compress ~source ~dest ~isize in
  let bytes_read = Lz4_string.uncompress ~source:dest ~dest:result_str
    ~osize:isize in

  printf "String tests: Original: %d, Compressed: %d, Perc: %d%%, Passed: %s\n"
    isize compressed_size (compressed_size * 100/isize)
    (Bool.to_string ((String.equal result_str source)
                     && (bytes_read = compressed_size)
                     && (compressed_size <= (compressBound isize))))
;;

let check_bigstring_compression source =
  let isize = Bigstring.length source in
  let dest = Lz4_bigstring.make_compression_dest source in
  let result_str = Bigstring.create isize in

  let compressed_size = Lz4_bigstring.compress ~source ~dest ~isize in
  let bytes_read = Lz4_bigstring.uncompress ~source:dest ~dest:result_str
    ~osize:isize in

  printf "Bigstring tests: Original: %d, Compressed: %d, Perc: %d%%, Passed: %s\n"
    isize compressed_size (compressed_size * 100/isize)
    (Bool.to_string ((result_str = source)
                     && (bytes_read = compressed_size)
                     && (compressed_size <= (compressBound isize))))
;;


let main () = begin
  check_string_compression str1;
  check_string_compression str2;
  check_string_compression str3;

  check_bigstring_compression bigstr1;
  check_bigstring_compression bigstr2;
  check_bigstring_compression bigstr3;
end

let () =
  Exn.handle_uncaught ~exit:true main

